#ifndef _MODULE_MARS_H_
#define _MODULE_MARS_H_
/*
 *                     Mars - Mars Library
 *
 *    Copyright (c) 1998-2015 by CryptoSoft GmbH. All Rights Reserved.
 *             http://www.cryptosoft.de, info@cryptosoft.de
 *
 *    Synopsis:	mars_init(key,len,pks)
 * Description: intializes all arrays and permutation tables for mars
 *       Input: pks  -  pointer to key schedule array
 *              key  -  Serpent key
 *              len  -  length of key in byte (max. 32)
 *      Output: 0 if OK; >0 if a (semi) weak was selected
 *
 *    Synopsis: mars_ecbYcode(in128,out128,pks)
 * Description: encrypts (Y=en) or decrypts (Y=de) 128-bit in to
 *              128-bit out using Serpent in ECB mode
 *     	 Input: in128  - pointer to 128-bit buffer of input data
 *              out128 - pointer to 128-bit buffer for output data
 *              pks   - pointer to key schedule array
 *      Output: 0 if OK
 *
 *    Synopsis: mars_cbcYcode(in128,out128,iv128,pks)
 * Description: encrypts (Y=en) or decrypts (Y=de) 128-bit in to
 *              128-bit out using Serpent in CBC mode
 *     	 Input: in128  - pointer to 128-bit buffer of input data
 *              out128 - pointer to 128-bit buffer for output data
 *              iv128  - pointer to 128-bit initialization vector
 *              pks   - pointer to key schedule array
 *      Output: 0 if OK
 *
 *    Synopsis: mars_cfbYcode(inpb,outpb,len,iv128,pks)
 * Description: encrypts (Y=en) or decrypts (Y=de) a given data block
 *              using Mars in 8-bit CFB mode
 *     	 Input: inpb  - pointer to buffer of input data
 *              outpb - pointer to buffer for output data
 *              len   - length of input buffer AND output buffer
 *              iv128 - pointer to 128-bit initialization vector
 *              pks   - pointer to key schedule array
 *      Output: 0 if OK
 *
 *    Synopsis: mars_ofbYcode(inpb,outpb,len,iv128,pks)
 * Description: encrypts (Y=en) or decrypts (Y=de) a given data block
 *              using Mars in 8-bit OFB mode
 *     	 Input: inpb  - pointer to buffer of input data
 *              outpb - pointer to buffer for output data
 *              len   - length of input buffer AND output buffer
 *              iv128 - pointer to 128-bit initialization vector
 *              pks   - pointer to key schedule array
 *      Output: 0 if OK
 *
 *
 */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct mars_schedule {
#ifdef X64
  unsigned int ks[40];
#else
  unsigned long ks[40];
#endif
} MARS_KS;

#if defined(WIN16)
#define DECL int far pascal
#define UCHAR unsigned char far
#define SIZE_T size_t far
#define MARS_PKS MARS_KS far *
#endif /* WIN16 */

#if defined(WIN32) || defined(OS2)
#define DECL int pascal
#define UCHAR unsigned char
#define SIZE_T size_t
#define MARS_PKS MARS_KS *
#endif /* WIN32 */

#if !defined(WIN16) && !defined(WIN32) && !defined(OS2)
#define DECL int
#define UCHAR unsigned char
#define SIZE_T size_t 
#define MARS_PKS MARS_KS *
#endif /* !WIN16 && !WIN32 && !OS2 */


DECL mars_init(UCHAR *key32, int len, MARS_PKS pks);
DECL mars_ecbencode(UCHAR *in128, UCHAR *out128, MARS_PKS pks);
DECL mars_ecbdecode(UCHAR *in128, UCHAR *out128, MARS_PKS pks);
DECL mars_cbcencode(UCHAR *in128, UCHAR *out128, UCHAR *iv128, MARS_PKS pks);
DECL mars_cbcdecode(UCHAR *in128, UCHAR *out128, UCHAR *iv128, MARS_PKS pks);
DECL mars_cfbencode(UCHAR *inpb, UCHAR *outpb, SIZE_T len, UCHAR *iv128, MARS_PKS pks);
DECL mars_cfbdecode(UCHAR *inpb, UCHAR *outpb, SIZE_T len, UCHAR *iv128, MARS_PKS pks);
DECL mars_ofbencode(UCHAR *inpb, UCHAR *outpb, SIZE_T len, UCHAR *iv128, MARS_PKS pks);
DECL mars_ofbdecode(UCHAR *inpb, UCHAR *outpb, SIZE_T len, UCHAR *iv128, MARS_PKS pks);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* _MODULE_MARS_H_ */
